<?php

if(!class_exists('IFWP')){
	final class IFWP {

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function after_setup_theme(){
			if(!defined('RWMB_VER')){
				ifwp_tgmpa([
					[
						'name' => 'Meta Box',
						'slug' => 'meta-box',
					],
				], [
					'id' => 'ifwp',
					'menu' => 'ifwp-install-plugins',
					'strings' => [
						'menu_title' => 'IFWP',
						'page_title' => __('One or more recommended modules are missing'), // One or more required modules are missing
					],
				]);
			}
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function plugins_loaded(){
			ifwp_add_larger_image_sizes();
			ifwp_fix_audio_video_ext();
			ifwp_hide_wp();
			ifwp_maybe_set_utm_params_cookies();
			add_action('wp_enqueue_scripts', [__CLASS__, 'wp_enqueue_scripts']);
			add_filter('sanitize_file_name', [__CLASS__, 'sanitize_file_name']);
			if(!defined('RWMB_VER')){
				return;
			}
			add_action('admin_enqueue_scripts', [__CLASS__, 'admin_enqueue_scripts']);
			add_action('init', [__CLASS__, 'init']);
			add_action('rwmb_meta_boxes', [__CLASS__, 'rwmb_meta_boxes']);
			add_action('wp_head', [__CLASS__, 'wp_head']);
			add_action('wp_print_footer_scripts', [__CLASS__, 'wp_print_footer_scripts']);
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static $admin_notices = [], $cloudinary_config = [], $cloudinary_image_sizes = [], $external_rules = [], $hooks = [], $image_sizes = [], $rewrite_rules = [], $tgmpa = [], $zoom_jwt = '';

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function admin_enqueue_scripts($hook_suffix){
			switch($hook_suffix){
				case 'post.php':
					if(!isset($_GET['action'], $_GET['post'])){
						return;
					}
					if('edit' !== $_GET['action']){
						return;
					}
					if(!in_array(get_post_type($_GET['post']), ['ifwp-css-code', 'ifwp-javascript-code'])){
						return;
					}
					break;
				case 'post-new.php':
					if(!isset($_GET['post_type'])){
						return;
					}
					if(!in_array($_GET['post_type'], ['ifwp-css-code', 'ifwp-javascript-code'])){
						return;
					}
					break;
				default:
					return;
			}
			wp_enqueue_script('ace', 'https://cdnjs.cloudflare.com/ajax/libs/ace/1.4.12/ace.min.js', ['jquery'], '1.4.12', true);
			wp_enqueue_script('ace-language-tools', 'https://cdnjs.cloudflare.com/ajax/libs/ace/1.4.12/ext-language_tools.min.js', ['ace'], '1.4.12', true);
			ifwp_enqueue_asset('ifwp-custom-code', 'custom-code.css');
			ifwp_enqueue_asset('ifwp-custom-code', 'custom-code.js', ['ace-language-tools']);
			wp_add_inline_script('ifwp-custom-code', 'ifwp_custom_code.load();');
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function init(){
			foreach([
				'css' => 'CSS',
				'javascript' => 'JavaScript',
			] as $mode => $label){
				register_post_type('ifwp-' . $mode . '-code', [
					'labels' => ifwp_post_type_labels('Custom ' . $label, 'Custom ' . $label, false),
					'show_in_admin_bar' => false,
					'show_in_menu' => 'themes.php',
					'show_ui' => true,
					'supports' => ['title'],
				]);
			}
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function rwmb_meta_boxes($meta_boxes){
			foreach([
				'css' => 'CSS',
				'javascript' => 'JavaScript',
			] as $mode => $label){
				$meta_boxes[] = [
					'fields' => [
						[
							'id' => 'ifwp_' . $mode . '_code',
							'sanitize_callback' => 'none',
							'type' => 'textarea',
						],
					],
					'post_types' => 'ifwp-' . $mode . '-code',
					'style' => 'seamless',
					'title' => 'Custom ' . $label,
				];
			}
			$meta_boxes[] = [
				'context' => 'side',
				'fields' => [
					[
						'admin_columns' => [
							'position' => 'after title',
							'title' => __('Published'),
						],
						'id' => 'ifwp_code_status',
						'std' => 1,
						'type' => 'switch',
					],
				],
				'post_types' => ['ifwp-css-code', 'ifwp-javascript-code'],
				'priority' => 'low',
				'title' => __('Published'),
			];
			return $meta_boxes;
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function sanitize_file_name($filename){
			return implode('.', array_map(function($piece){
				return preg_replace('/[^A-Za-z0-9_-]/', '', $piece);
			}, explode('.', $filename)));
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function wp_enqueue_scripts(){
			if(!wp_script_is('jquery')){
				wp_enqueue_script('jquery');
			}
			ifwp_enqueue_asset('ifwp', 'functions.js', ['jquery']);
			$current_utm_params = ifwp_get_current_utm_params();
			$current_utm_params_with_values = [];
			foreach($current_utm_params as $key => $value){
				if($value){
					$current_utm_params_with_values[] = $key . '=' . $value;
				}
			}
			$current_utm_params_str = implode('&', $current_utm_params_with_values);
			$current_utm_params['str'] = $current_utm_params_str;
			wp_localize_script('ifwp', 'ifwp_current_utm_params', $current_utm_params);
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function wp_head(){
			$posts = get_posts([
				'meta_query' => [
					[
						'key' => 'ifwp_code_status',
						'value' => 1,
					],
				],
				'post_type' => 'ifwp-css-code',
				'posts_per_page' => -1,
			]);
			if($posts){
				echo '<style id="ifwp-custom-css">';
				foreach($posts as $post){
					if(apply_filters('ifwp_css_code', true, $post)){
						echo "\n" . '/* ' . esc_html($post->post_title) . '*/' . "\n";
						echo get_post_meta($post->ID, 'ifwp_css_code', true);
					}
				}
				echo '</style>';
			}
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		public static function wp_print_footer_scripts(){
			$posts = get_posts([
				'meta_query' => [
					[
						'key' => 'ifwp_code_status',
						'value' => 1,
					],
				],
				'post_type' => 'ifwp-javascript-code',
				'posts_per_page' => -1,
			]);
			if($posts){
				echo '<script id="ifwp-custom-javascript">';
				foreach($posts as $post){
					if(apply_filters('ifwp_javascript_code', true, $post)){
						echo "\n" . '/* ' . esc_html($post->post_title) . '*/' . "\n";
						echo trim(get_post_meta($post->ID, 'ifwp_javascript_code', true)) . "\n";
					}
				}
				echo '</script>';
			}
		}

		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	}
}
