<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_basename')){
    function ifwp_basename($filename = ''){
        return preg_replace('/\?.*/', '', wp_basename($filename));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_download')){
    function ifwp_download($url = '', $post_id = 0, $timeout = 300){
        $upload_dir = wp_upload_dir();
        $path = $upload_dir['path'];
        $filename = ifwp_basename($url);
        $filename = wp_unique_filename($path, $filename);
        $args = [
            'filename' => trailingslashit($path) . $filename,
            'timeout' => $timeout,
        ];
        $file = ifwp_remote_download($url, $args);
        if(is_wp_error($file)){
            return $file;
        }
        return ifwp_sideload($file, $post_id);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_download_dir')){
    function ifwp_download_dir(){
        $upload_dir = wp_get_upload_dir();
        $dir = $upload_dir['basedir'] . '/ifwp-downloads';
        if(!wp_mkdir_p($dir)){
            return ifwp_error(__('Could not create directory.'));
        }
        if(!wp_is_writable($dir)){
            return ifwp_error(__('Destination directory for file streaming does not exist or is not writable.'));
        }
        return $dir;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_download_url')){
    function ifwp_download_url($file = ''){
        $upload_dir = wp_get_upload_dir();
        if(empty($file)){
            return $upload_dir['baseurl'] . '/ifwp-downloads';
        }
        if(!ifwp_in_uploads_dir($file)){
            return ifwp_error(sprintf(__('Unable to locate needed folder (%s).'), __('The uploads directory')));
        }
        return str_replace($upload_dir['basedir'], $upload_dir['baseurl'], $file);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_filesystem')){
    function ifwp_filesystem(){
        global $wp_filesystem;
    	if($wp_filesystem instanceof WP_Filesystem_Direct){
    		return $wp_filesystem;
    	}
    	if(!function_exists('get_filesystem_method')){
    		require_once(ABSPATH . 'wp-admin/includes/file.php');
    	}
    	if('direct' !== get_filesystem_method()){
    		return ifwp_error(__('Could not access filesystem.'));
    	}
    	if(!WP_Filesystem()){
    		return ifwp_error(__('Filesystem error.'));
    	}
    	return $wp_filesystem;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_move_uploaded_file')){
    function ifwp_move_uploaded_file($tmp_name = '', $path = ''){
		$fs = ifwp_filesystem();
        if(is_wp_error($fs)){
            return $fs;
        }
		if(!$fs->exists($tmp_name)){
            return ifwp_error(__('File doesn&#8217;t exist?'), $tmp_name); // __('File does not exist! Please double check the name and try again.')
        }
		if($path){
			if(!ifwp_in_uploads_dir($path)){
				return ifwp_error(sprintf(__('Unable to locate needed folder (%s).'), __('The uploads directory')));
			}
			$path = untrailingslashit($path);
		} else {
			$upload_dir = wp_upload_dir();
			$path = $upload_dir['path'];
		}
		$original_filename = wp_basename($tmp_name);
		$filename = wp_unique_filename($path, $original_filename);
		$file = trailingslashit($path) . $filename;
		if(!$fs->copy($tmp_name, $file)){
			return ifwp_error(sprintf(__('The uploaded file could not be moved to %s.'), $file));
		}
		return $file;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_sideload')){
    function ifwp_sideload($file = '', $post_id = 0){
        $fs = ifwp_filesystem();
        if(is_wp_error($fs)){
            return $fs;
        }
        if(!$fs->exists($file)){
            return ifwp_error(__('File doesn&#8217;t exist?'), $file);
        }
        if(!ifwp_in_uploads_dir($file)){
            return ifwp_error(sprintf(__('Unable to locate needed folder (%s).'), __('The uploads directory')));
        }
        $filename = wp_basename($file);
        $filetype_and_ext = wp_check_filetype_and_ext($file, $filename);
        if(!$filetype_and_ext['type']){
            return ifwp_error(__('Sorry, this file type is not permitted for security reasons.'));
        }
        $attachment_id = wp_insert_attachment([
            'guid' => ifwp_download_url($file),
            'post_mime_type' => $filetype_and_ext['type'],
            'post_status' => 'inherit',
            'post_title' => preg_replace('/\.[^.]+$/', '', $filename),
        ], $file, $post_id, true);
        if(is_wp_error($attachment_id)){
            return $attachment_id;
        }
        ifwp_maybe_generate_attachment_metadata($attachment_id);
        return $attachment_id;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_in_uploads_url')){
    function ifwp_in_uploads_url($url = ''){
        $upload_dir = wp_get_upload_dir();
        return (0 === strpos($url, $upload_dir['baseurl']));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_in_uploads_dir')){
    function ifwp_in_uploads_dir($file = ''){
        $upload_dir = wp_get_upload_dir();
        return (0 === strpos($file, $upload_dir['basedir']));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_upload')){
    function ifwp_upload($tmp_name = '', $post_id = 0, $path = ''){
        $file = ifwp_move_uploaded_file($tmp_name, $path);
		if(is_wp_error($file)){
			return $file;
		}
		return ifwp_sideload($file, $post_id);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
