<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_the_dashboard')){
    function ifwp_hide_the_dashboard($capability = 'edit_posts', $location = ''){
        ifwp_one('admin_init', function() use($capability, $location){
            if(wp_doing_ajax() or current_user_can($capability)){
                return;
            }
            if(false === filter_var($location, FILTER_VALIDATE_URL)){
                $location = home_url();
            }
            wp_safe_redirect($location);
            exit;
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_the_toolbar')){
    function ifwp_hide_the_toolbar($capability = 'edit_posts'){
        ifwp_one('show_admin_bar', function($show) use($capability){
            if(!current_user_can($capability)){
				$show = false;
			}
			return $show;
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_others_media')){
    function ifwp_hide_others_media($capability = 'edit_others_posts'){
        ifwp_one('ajax_query_attachments_args', function($query) use($capability){
            if(!current_user_can($capability)){
				$query['author'] = get_current_user_id();
			}
			return $query;
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_others_posts')){
    function ifwp_hide_others_posts($capability = 'edit_others_posts'){
        ifwp_one('current_screen', function($current_screen) use($capability){
            global $pagenow;
            if('edit.php' !== $pagenow or current_user_can($capability)){
                return;
            }
            ifwp_on('views_' . $current_screen->id, function($views){
                foreach($views as $index => $view){
                    $views[$index] = preg_replace('/ <span class="count">\([0-9]+\)<\/span>/', '', $view);
                }
                return $views;
            });
        });
        ifwp_one('pre_get_posts', function($query) use($capability){
            global $pagenow;
            if('edit.php' !== $pagenow){
                return $query;
            }
            if(!current_user_can($capability)){
                $query->set('author', get_current_user_id());
            }
            return $query;
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_the_rest_api')){
    function ifwp_hide_the_rest_api($capability = 'read'){
        ifwp_one('rest_authentication_errors', function($error) use($capability){
            if(!empty($error)){
				return $error;
			}
            if(!current_user_can($capability)){
				return ifwp_error(__('You need a higher level of permission.'), [
					'status' => 401,
				]);
			}
			return null;
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_the_entire_site')){
    function ifwp_hide_the_entire_site($capability = 'read', $exclude_other_pages = [], $exclude_special_pages = []){
        ifwp_one('template_redirect', function() use($capability, $exclude_other_pages, $exclude_special_pages){
            if(in_array(get_the_ID(), (array) $exclude_other_pages)){
                return;
            }
            if(is_front_page() and in_array('front_end', (array) $exclude_special_pages)){
                return;
            }
            if(is_home() and in_array('home', (array) $exclude_special_pages)){
                return;
            }
            if(!is_user_logged_in()){
                auth_redirect();
            }
            if(!current_user_can($capability)){
                wp_die('<h1>' . __('You need a higher level of permission.') . '</h1>' . '<p>' . __('Sorry, you are not allowed to access this page.') . '</p>', 403);
            }
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_uploads_subdir')){
    function ifwp_hide_uploads_subdir($subdir = ''){
		if(is_multisite()){
			return;
		}
		$plugin_file = ifwp_plugin_file();
		if(!$plugin_file){
			return ifwp_error(sprintf(__('Unable to locate needed folder (%s).'), __('The plugins directory')));
		}
		$uploads_use_yearmonth_folders = false;
		$subdir = ltrim(untrailingslashit(trim($subdir)), '/');
		if($subdir){
			$subdir = '/' . $subdir;
		} else {
			$subdir = '';
			if(get_option('uploads_use_yearmonth_folders')){
				$subdir = '/(\d{4})/(\d{2})';
				$uploads_use_yearmonth_folders = true;
			}
		}
		$wp_upload_dir = wp_get_upload_dir();
		$regex = $wp_upload_dir['baseurl'] . $subdir. '/(.+)';
		$query = plugin_dir_url($plugin_file) . 'assets/readfile.php';
		if($uploads_use_yearmonth_folders){
			$atts['yyyy'] = '$1';
			$atts['mm'] = '$2';
			$atts['file'] = '$3';
		} else {
			$atts['file'] = '$1';
		}
		$query = add_query_arg($atts, $query);
		ifwp_add_external_rule($regex, $query);
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_hide_wp')){
    function ifwp_hide_wp(){
		ifwp_local_login_header();
		ifwp_one('wp_before_admin_bar_render', function(){
			global $wp_admin_bar;
			$wp_admin_bar->remove_node('wp-logo');
		});
        remove_action('welcome_panel', 'wp_welcome_panel');
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
