<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_add_admin_notice')){
    function ifwp_add_admin_notice($message = '', $class = 'warning', $is_dismissible = false){
        $html = ifwp_admin_notice_html($message, $class, $is_dismissible);
        $md5 = md5($html);
        IFWP::$admin_notices[$md5] = $html;
        ifwp_one('admin_notices', function(){
            foreach(IFWP::$admin_notices as $admin_notice){
                echo $admin_notice;
            }
        });
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_admin_notice_html')){
    function ifwp_admin_notice_html($message = '', $class = 'warning', $is_dismissible = false){
        if(!in_array($class, ['error', 'info', 'success', 'warning'])){
            $class = 'warning';
        }
        if($is_dismissible){
            $class .= ' is-dismissible';
        }
        return '<div class="notice notice-' . $class . '"><p>' . $message . '</p></div>';
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_array_keys_exist')){
    function ifwp_array_keys_exist($keys = [], $array = []){
        if(!is_array($keys) or !is_array($array)){
            return false;
        }
        foreach($keys as $key){
            if(!array_key_exists($key, $array)){
                return false;
            }
        }
        return true;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_base64_urldecode')){
    function ifwp_base64_urldecode($data = '', $strict = false){
        return base64_decode(strtr($data, '-_', '+/'), $strict);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_base64_urlencode')){
    function ifwp_base64_urlencode($data = ''){
        return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_clone_role')){
    function ifwp_clone_role($source = '', $destination = '', $display_name = ''){
        $role = get_role($source);
        if(is_null($role)){
            return null;
        }
        $destination = ifwp_canonicalize($destination);
        return add_role($destination, $display_name, $role->capabilities);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_current_screen_in')){
    function ifwp_current_screen_in($ids = []){
        global $current_screen;
        if(!is_array($ids)){
            return false;
        }
        if(!isset($current_screen)){
            return false;
        }
        return in_array($current_screen->id, $ids);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_current_screen_is')){
    function ifwp_current_screen_is($id = ''){
        global $current_screen;
        if(!is_string($id)){
            return false;
        }
        if(!isset($current_screen)){
            return false;
        }
        return ($current_screen->id === $id);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_custom_login_logo')){
    function ifwp_custom_login_logo($attachment_id = 0, $half = true){
        if(!wp_attachment_is_image($attachment_id)){
            return ifwp_error(__('File is not an image.'));
        }
        ifwp_one('login_enqueue_scripts', function() use($attachment_id, $half){
            $custom_logo = wp_get_attachment_image_src($attachment_id, 'medium');
            $height = $custom_logo[2];
            $width = $custom_logo[1];
            if($half){
                $height = $height / 2;
                $width = $width / 2;
            } ?>
            <style type="text/css">
                #login h1 a,
                .login h1 a {
                    background-image: url(<?php echo $custom_logo[0]; ?>);
                    background-size: <?php echo $width; ?>px <?php echo $height; ?>px;
                    height: <?php echo $height; ?>px;
                    width: <?php echo $width; ?>px;
                }
            </style><?php
		});
        return true;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_error')){
    function ifwp_error($message = '', $data = ''){
        if(is_wp_error($message)){
            $data = $message->get_error_data();
            $message = $message->get_error_message();
        }
        if(empty($message)){
            $message = __('Something went wrong.');
        }
        return new WP_Error('ifwp_error', $message, $data);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_fa_file_type')){
    function ifwp_fa_file_type($post = null){
        if('attachment' !== get_post_status($post)){
			return '';
		}
		if(wp_attachment_is('audio', $post)){
			return 'audio';
		}
		if(wp_attachment_is('image', $post)){
			return 'image';
		}
		if(wp_attachment_is('video', $post)){
			return 'video';
		}
		$type = get_post_mime_type($post);
		switch($type){
			case 'application/zip':
			case 'application/x-rar-compressed':
			case 'application/x-7z-compressed':
			case 'application/x-tar':
				return 'archive';
				break;
			case 'application/vnd.ms-excel':
			case 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':
				return 'excel';
				break;
			case 'application/pdf':
				return 'pdf';
				break;
			case 'application/vnd.ms-powerpoint':
			case 'application/vnd.openxmlformats-officedocument.presentationml.presentation':
				return 'powerpoint';
				break;
			case 'application/msword':
			case 'application/vnd.openxmlformats-officedocument.wordprocessingml.document':
				return 'word';
				break;
			default:
				return 'file';
		}
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_get_memory_size')){
    function ifwp_get_memory_size(){
        if(!function_exists('exec')){
            return 0;
        }
        exec('free -b', $output);
        $output = sanitize_text_field($output[1]);
        $output = explode(' ', $output);
        return (int) $output[1];
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_get_posts_query')){
    function ifwp_get_posts_query($args = null){
        $defaults = [
			'numberposts' => 5,
			'category' => 0,
			'orderby' => 'date',
			'order' => 'DESC',
			'include' => [],
			'exclude' => [],
			'meta_key' => '',
			'meta_value' => '',
			'post_type' => 'post',
			'suppress_filters' => true,
		];
		$parsed_args = wp_parse_args( $args, $defaults );
		if(empty($parsed_args['post_status'])){
			$parsed_args['post_status'] = ('attachment' === $parsed_args['post_type']) ? 'inherit' : 'publish';
		}
		if(!empty($parsed_args['numberposts']) and empty($parsed_args['posts_per_page'])){
			$parsed_args['posts_per_page'] = $parsed_args['numberposts'];
		}
		if(!empty($parsed_args['category'])){
			$parsed_args['cat'] = $parsed_args['category'];
		}
		if(!empty($parsed_args['include'])){
			$incposts = wp_parse_id_list($parsed_args['include']);
			$parsed_args['posts_per_page'] = count( $incposts );  // Only the number of posts included.
			$parsed_args['post__in'] = $incposts;
		} elseif(!empty($parsed_args['exclude'])){
			$parsed_args['post__not_in'] = wp_parse_id_list($parsed_args['exclude']);
		}
		$parsed_args['ignore_sticky_posts'] = true;
		$parsed_args['no_found_rows'] = true;
		$query = new WP_Query;
		$query->query($parsed_args);
		return $query;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_format_function')){
    function ifwp_format_function($function_name = '', $args = []){
        $str = '<div style="color: #24831d; font-family: monospace; font-weight: 400;">' . $function_name . '(';
        $function_args = [];
        foreach($args as $arg){
            $arg = shortcode_atts([
                'default' => 'null',
                'name' => '',
                'type' => '',
            ], $arg);
            if($arg['default'] and $arg['name'] and $arg['type']){
                $function_args[] = '<span style="color: #cd2f23; font-family: monospace; font-style: italic; font-weight: 400;">' . $arg['type'] . '</span> <span style="color: #0f55c8; font-family: monospace; font-weight: 400;">$' . $arg['name'] . '</span> = <span style="color: #000; font-family: monospace; font-weight: 400;">' . $arg['default'] . '</span>';
            }
        }
        if($function_args){
            $str .= ' ' . implode(', ', $function_args) . ' ';
        }
        $str .= ')</div>';
		return $str;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_is_array_assoc')){
    function ifwp_is_array_assoc($array = []){
        if(!is_array($array)){
            return false;
        }
        $end = count($array) - 1;
        if(array_keys($array) === range(0, $end)){
            return false;
        }
        return true;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_is_doing_heartbeat')){
    function ifwp_is_doing_heartbeat(){
        return (defined('DOING_AJAX') and DOING_AJAX and isset($_POST['action']) and 'heartbeat' === $_POST['action']);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_is_google_workspace')){
    function ifwp_is_google_workspace($email = ''){
		if(!is_email($email)){
			return false;
		}
		list($local, $domain) = explode('@', $email, 2);
		if('gmail.com' === strtolower($domain)){
			return true;
		}
		getmxrr($domain, $mxhosts);
		return in_array('aspmx.l.google.com', $mxhosts);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_is_post_revision_or_auto_draft')){
    function ifwp_is_post_revision_or_auto_draft($post = null){
        return (wp_is_post_revision($post) or 'auto-draft' === get_post_status($post));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_ksort_deep')){
    function ifwp_ksort_deep($array = []){
        if(!ifwp_is_array_assoc($array)){
            return $array;
        }
        ksort($array);
        foreach($array as $key => $value){
            $array[$key] = ifwp_ksort_deep($value);
        }
        return $array;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_list_pluck')){
    function ifwp_list_pluck($list = [], $index_key = ''){
		$newlist = [];
		foreach($list as $value){
			if(is_object($value)){
				if(isset($value->$index_key)){
					$newlist[$value->$index_key] = $value;
				} else {
					$newlist[] = $value;
				}
			} else {
				if(isset($value[$index_key])){
					$newlist[$value[$index_key]] = $value;
				} else {
					$newlist[] = $value;
				}
			}
		}
		return $newlist;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_local_login_header')){
    function ifwp_local_login_header(){
        ifwp_one('login_headertext', function($login_header_text){
			return get_option('blogname');
		});
		ifwp_one('login_headerurl', function($login_header_url){
			return home_url();
		});
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_post_type_labels')){
    function ifwp_post_type_labels($singular = '', $plural = '', $all = true){
        if(empty($singular)){
            return [];
        }
        if(empty($plural)){
            $plural = $singular;
        }
        return [
            'name' => $plural,
            'singular_name' => $singular,
            'add_new' => 'Add New',
            'add_new_item' => 'Add New ' . $singular,
            'edit_item' => 'Edit ' . $singular,
            'new_item' => 'New ' . $singular,
            'view_item' => 'View ' . $singular,
            'view_items' => 'View ' . $plural,
            'search_items' => 'Search ' . $plural,
            'not_found' => 'No ' . strtolower($plural) . ' found.',
            'not_found_in_trash' => 'No ' . strtolower($plural) . ' found in Trash.',
            'parent_item_colon' => 'Parent ' . $singular . ':',
            'all_items' => ($all ? 'All ' : '') . $plural,
            'archives' => $singular . ' Archives',
            'attributes' => $singular . ' Attributes',
            'insert_into_item' => 'Insert into ' . strtolower($singular),
            'uploaded_to_this_item' => 'Uploaded to this ' . strtolower($singular),
            'featured_image' => 'Featured image',
            'set_featured_image' => 'Set featured image',
            'remove_featured_image' => 'Remove featured image',
            'use_featured_image' => 'Use as featured image',
            'filter_items_list' => 'Filter ' . strtolower($plural) . ' list',
            'items_list_navigation' => $plural . ' list navigation',
            'items_list' => $plural . ' list',
            'item_published' => $singular . ' published.',
            'item_published_privately' => $singular . ' published privately.',
            'item_reverted_to_draft' => $singular . ' reverted to draft.',
            'item_scheduled' => $singular . ' scheduled.',
            'item_updated' => $singular . ' updated.',
        ];
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_seems_false')){
    function ifwp_seems_false($data = ''){
		return in_array((string) $data, ['0', 'false', 'off'], true);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_seems_mysql_date')){
    function ifwp_seems_mysql_date($pattern = ''){
        return preg_match('/^\d{4}\-\d{2}\-\d{2} \d{2}:\d{2}:\d{2}$/', $pattern);
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_seems_true')){
    function ifwp_seems_true($data = ''){
        return in_array((string) $data, ['1', 'on', 'true'], true);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_signon')){
    function ifwp_signon($username_or_email = '', $password = '', $remember = false){
        if(is_user_logged_in()){
            return wp_get_current_user();
        } else {
			$fix = ifwp_on('wordfence_ls_require_captcha', function($required){
                return false;
            });
            $user = wp_signon([
                'remember' => $remember,
                'user_login' => $username_or_email,
                'user_password' => $password,
            ]);
			ifwp_off('wordfence_ls_require_captcha', $fix);
			if(is_wp_error($user)){
				return $user;
			}
            return wp_set_current_user($user->ID);
        }
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_signon_without_password')){
    function ifwp_signon_without_password($username_or_email = '', $remember = false){
        if(is_user_logged_in()){
            return wp_get_current_user();
        } else {
            $idx = ifwp_on('authenticate', function($user, $username_or_email){
                if(is_null($user)){
                    if(is_email($username_or_email)){
                        $user = get_user_by('email', $username_or_email);
                    }
                    if(is_null($user)){
                        $user = get_user_by('login', $username_or_email);
                    }
                }
                return $user;
            }, 10, 2);
			$fix = ifwp_on('wordfence_ls_require_captcha', function($required){
                return false;
            });
            $user = wp_signon([
                'remember' => $remember,
                'user_login' => $username_or_email,
                'user_password' => '',
            ]);
			ifwp_off('wordfence_ls_require_captcha', $fix);
            ifwp_off('authenticate', $idx);
			if(is_wp_error($user)){
				return $user;
			}
            return wp_set_current_user($user->ID);
        }
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
