<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_are_plugins_active')){
    function ifwp_are_plugins_active($plugins = []){
        if(!is_array($plugins)){
            return false;
        }
        foreach($plugins as $plugin){
            if(!ifwp_is_plugin_active($plugin)){
                return false;
            }
        }
        return true;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_enqueue_asset')){
    function ifwp_enqueue_asset($handle = '', $src = '', $deps = []){
		$debug = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
		$plugin_file = ifwp_plugin_file($debug[0]['file']);
		if(!$plugin_file){
			return ifwp_error(sprintf(__('Unable to locate needed folder (%s).'), __('The plugins directory')));
		}
        $filename = basename($src);
		$file = plugin_dir_path($plugin_file) . 'assets/' . $filename;
		if(!file_exists($file)){
			return ifwp_error(__('File doesn&#8217;t exist?'), $file); // __('File does not exist! Please double check the name and try again.')
		}
		$src = plugin_dir_url($plugin_file) . 'assets/' . $filename;
		$ver = filemtime($file);
		return ifwp_enqueue($handle, $src, $deps, $ver, true);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_is_plugin_active')){
    function ifwp_is_plugin_active($plugin = ''){
        if(!function_exists('is_plugin_active')){
            require_once(ABSPATH . 'wp-admin/includes/plugin.php');
        }
        return is_plugin_active($plugin);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_is_plugin_deactivating')){
    function ifwp_is_plugin_deactivating($file = ''){
        global $pagenow;
        if(!$file){
			$debug = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
			$file = ifwp_plugin_file($debug[0]['file']);
			if(!$file){
				return false;
			}
        }
        if(!is_file($file)){
            return false;
        }
        return (is_admin() and 'plugins.php' === $pagenow and isset($_GET['action'], $_GET['plugin']) and 'deactivate' === $_GET['action'] and plugin_basename($file) === $_GET['plugin']);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_plugin_file')){
	function ifwp_plugin_file($file = ''){
		global $wp_plugin_paths;
		if(!$file){
			$debug = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
			$file = $debug[0]['file'];
		}
		$file = wp_normalize_path($file);
		arsort($wp_plugin_paths);
		foreach($wp_plugin_paths as $dir => $realdir){
			if(strpos($file, $realdir) === 0){
				$file = $dir . substr($file, strlen($realdir));
			}
		}
		$plugin_dir = wp_normalize_path(WP_PLUGIN_DIR);
		$mu_plugin_dir = wp_normalize_path(WPMU_PLUGIN_DIR);
		if(!preg_match('#^' . preg_quote($plugin_dir, '#') . '/|^' . preg_quote($mu_plugin_dir, '#') . '/#', $file)){
			return '';
		}
		if(preg_match('#^' . preg_quote($plugin_dir, '#') . '/#', $file)){
			$dir = $plugin_dir;
		} else {
			$dir = $mu_plugin_dir;
		}
		$file = preg_replace('#^' . preg_quote($plugin_dir, '#') . '/|^' . preg_quote($mu_plugin_dir, '#') . '/#', '', $file);
		$file = trim($file, '/');
		if(strpos($file, '/') === false){
			return '';
		}
		$parts = explode('/', $file);
		$part = $parts[0] . '/';
		$active_plugins = (array) get_option('active_plugins', []);
		foreach($active_plugins as $active_plugin){
			if(strpos($active_plugin, $part) === 0){
				return $dir . '/' . $active_plugin;
			}
		}
		$active_sitewide_plugins = (array) get_site_option('active_sitewide_plugins', []);
		$active_sitewide_plugins = array_keys($active_sitewide_plugins);
		foreach($active_sitewide_plugins as $active_sitewide_plugin){
			if(strpos($active_sitewide_plugin, $part) === 0){
				return $dir . '/' . $active_sitewide_plugin;
			}
		}
		return '';
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_plugin_load')){
	function ifwp_plugin_load($namespace = ''){
		if(did_action('ifwp_after_setup_theme') or !did_action('ifwp_plugins_loaded')){
			return;
		}
		$debug = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
		$plugin_file = ifwp_plugin_file($debug[0]['file']);
		if(!$plugin_file){
			return;
		}
		$dirname = strtolower(ltrim(dirname(plugin_basename($plugin_file)), '/'));
		if(did_action($dirname . '_loaded')){
			return;
		}
		$dir = plugin_dir_path($plugin_file);
		ifwp_plugin_load_functions($dir, $namespace);
		ifwp_plugin_load_classes($dir, $namespace);
		$dir = get_stylesheet_directory(). '/' . $dirname;
		ifwp_plugin_load_functions($dir, $namespace);
		ifwp_plugin_load_classes($dir, $namespace);
		do_action($dirname . '_loaded');
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_plugin_load_classes')){
	function ifwp_plugin_load_classes($dir = '', $namespace = ''){
		$dir = trailingslashit($dir) . 'classes';
		if(@is_dir($dir)){
			foreach(glob($dir . '/*.php') as $file){
				require_once($file);
				$class = ifwp_canonicalize(basename($file, '.php'));
				if($namespace){
					$class = $namespace . '\\' . $class;
				}
				if(is_callable([$class, 'ifwp_plugins_loaded'])){
					call_user_func([$class, 'ifwp_plugins_loaded']);
				}
				if(is_callable([$class, 'ifwp_after_setup_theme'])){
					add_action('ifwp_after_setup_theme', [$class, 'ifwp_after_setup_theme']);
				}
			}
		}
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_plugin_load_functions')){
	function ifwp_plugin_load_functions($dir = ''){
		$dir = trailingslashit($dir) . 'functions';
		if(@is_dir($dir)){
			foreach(glob($dir . '/*.php') as $file){
				require_once($file);
			}
		}
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
