<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_country')){
    function ifwp_remote_country(){
        switch(true){
			case isset($_SERVER['HTTP_CF_IPCOUNTRY']):
				$country = $_SERVER['HTTP_CF_IPCOUNTRY'];
				break;
			case class_exists('wfUtils'):
				$country = wfUtils::IP2Country(ifwp_remote_ip());
				break;
			default:
				$country = '';
		}
    	return strtoupper($country);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_delete')){
    function ifwp_remote_delete($url = '', $args = []){
        return ifwp_remote_request('DELETE', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_download')){
    function ifwp_remote_download($url = '', $args = []){
        $args = wp_parse_args($args, [
            'filename' => '',
            'timeout' => 300,
        ]);
        if(empty($args['filename'])){
            $download_dir = ifwp_download_dir();
            if(is_wp_error($download_dir)){
                return $download_dir;
            }
    		$filename = ifwp_basename($url);
    		$filename = wp_unique_filename($download_dir, $filename);
            $args['filename'] = trailingslashit($download_dir) . $filename;
        } else {
            if(!ifwp_in_uploads_dir($args['filename'])){
                return ifwp_error(sprintf(__('Unable to locate needed folder (%s).'), __('The uploads directory')));
            }
        }
        $args['stream'] = true;
        $args['timeout'] = ifwp_sanitize_timeout($args['timeout']);
        $response = ifwp_remote_request('GET', $url, $args);
        if(is_wp_error($response)){
            @unlink($args['filename']);
            return $response;
        }
        return $args['filename'];
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_get')){
    function ifwp_remote_get($url = '', $args = []){
        return ifwp_remote_request('GET', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_head')){
    function ifwp_remote_head($url = '', $args = []){
        return ifwp_remote_request('HEAD', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_ip')){
    function ifwp_remote_ip(){
        switch(true){
			case isset($_SERVER['HTTP_CF_CONNECTING_IP']):
				$ip = $_SERVER['HTTP_CF_CONNECTING_IP'];
				break;
			case class_exists('wfUtils'):
				$ip = wfUtils::getIP();
				break;
			case isset($_SERVER['HTTP_X_FORWARDED_FOR']):
				$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
				break;
			case isset($_SERVER['HTTP_X_REAL_IP']):
				$ip = $_SERVER['HTTP_X_REAL_IP'];
				break;
			case isset($_SERVER['REMOTE_ADDR']):
				$ip = $_SERVER['REMOTE_ADDR'];
				break;
			default:
				$ip = '';
		}
		if(false !== strpos($ip, ',')){
			$ip = explode(',', $ip);
			$ip = array_map('trim', $ip);
			$ip = array_filter($ip);
			$ip = array_shift($ip);
		}
    	if(!WP_Http::is_ip_address($ip)){
    		return '';
    	}
    	return $ip;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_lib')){
    function ifwp_remote_lib($url = '', $expected_dir = ''){
        $download_dir = ifwp_download_dir();
    	if(is_wp_error($download_dir)){
    		return $download_dir;
    	}
    	$fs = ifwp_filesystem();
    	if(is_wp_error($fs)){
    		return $fs;
    	}
    	$name = 'ifwp_remote_lib_' . md5($url);
    	$to = trailingslashit($download_dir) . $name;
    	if(empty($expected_dir)){
    		$expected_dir = $to;
    	} else {
    		$expected_dir = ltrim($expected_dir, '/');
    		$expected_dir = untrailingslashit($expected_dir);
    		$expected_dir = trailingslashit($to) . $expected_dir;
    	}
		/*$dir = get_option($name);
        if($dir and !$fs->is_dir($dir)){
            delete_option($name);
        }*/
    	/*$dir = get_option($name);
    	$dirlist = $fs->dirlist($expected_dir, false);
    	if(empty($dir)){
    		if(!empty($dirlist)){
    			return ifwp_error(__('Destination folder already exists.'));
    		}
    	} else {
    		if(empty($dirlist)){
    			delete_option($name);
    		} else {
    			return $dir;
    		}
    	}*/
		// To do: Si el folder existe pero la opción no, borre el folder. Si la opción existe pero el el folder no, borre la opción.
		$dirlist = $fs->dirlist($expected_dir, false);
        if(!empty($dirlist)){
            return $expected_dir;
        }
    	$file = ifwp_remote_download($url);
    	if(is_wp_error($file)){
    		return $file;
    	}
    	$result = unzip_file($file, $to);
    	@unlink($file);
    	if(is_wp_error($result)){
    		$fs->rmdir($to, true);
    		return $result;
    	}
    	if(!$fs->dirlist($expected_dir, false)){
    		return ifwp_error(__('Destination directory for file streaming does not exist or is not writable.'));
    	}
    	//update_option($name, $expected_dir);
    	return $expected_dir;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_options')){
    function ifwp_remote_options($url = '', $args = []){
        return ifwp_remote_request('OPTIONS', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_patch')){
    function ifwp_remote_patch($url = '', $args = []){
        return ifwp_remote_request('PATCH', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_post')){
    function ifwp_remote_post($url = '', $args = []){
        return ifwp_remote_request('POST', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_put')){
    function ifwp_remote_put($url = '', $args = []){
        return ifwp_remote_request('PUT', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_request')){
	function ifwp_remote_request($method = '', $url = '', $args = []){
		$args = ifwp_sanitize_remote_args($args);
		$args['method'] = strtoupper($method);
		if(!ifwp_seems_wp_http_request($args, true)){
			return ifwp_error(__('Invalid request method.'));
		}
		if(empty($args['user-agent']) and !empty($_SERVER['HTTP_USER_AGENT'])){
			$args['user-agent'] = $_SERVER['HTTP_USER_AGENT'];
		}
		$response = wp_remote_request($url, $args);
		if(is_wp_error($response)){
			return $response;
		}
		$body = wp_remote_retrieve_body($response);
		$code = wp_remote_retrieve_response_code($response);
		$headers = wp_remote_retrieve_headers($response);
		$request = new WP_REST_Request($method);
		$request->set_body($body);
		$request->set_headers($headers);
		$is_valid = $request->has_valid_params();
		if(is_wp_error($is_valid)){
			return $is_valid;
		}
		$is_json = $request->is_json_content_type();
		if($code >= 200 and $code < 300){
			if($is_json){
				return $request->get_json_params();
			}
			return $body;
		}
		if($is_json){
			$error = ifwp_seems_wp_error($request->get_json_params());
			if(is_wp_error($error)){
				return $error;
			}
		}
		$message = wp_remote_retrieve_response_message($response);
		if(empty($message)){
			$message = get_status_header_desc($code);
		}
		if(empty($message)){
			$message = __('Something went wrong.');
		}
		return ifwp_error($message, [
			'body' => $body,
			'headers' => $headers,
			'status' => $code,
		]);
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_remote_trace')){
    function ifwp_remote_trace($url = '', $args = []){
        return ifwp_remote_request('TRACE', $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_sanitize_remote_args')){
    function ifwp_sanitize_remote_args($args = []){
        if(!is_array($args)){
    		$args = wp_parse_args($args);
    	}
        if(ifwp_seems_wp_http_request($args)){
            return $args;
        }
    	return [
    		'body' => $args,
    	];
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_sanitize_timeout')){
    function ifwp_sanitize_timeout($timeout = 0){
        $timeout = absint($timeout);
        $max_execution_time = (int) ini_get('max_execution_time');
        if(0 !== $max_execution_time){
            if(0 === $timeout or $timeout > $max_execution_time){
                $timeout = $max_execution_time - 1;
            }
        }
        if(isset($_SERVER['HTTP_CF_RAY'])){
            if(0 === $timeout or $timeout > 99){
                $timeout = 99;
            }
        }
        return $timeout;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_seems_wp_http_request')){
    function ifwp_seems_wp_http_request($args = [], $method_verification = false){
        if(!is_array($args)){
    		return false;
    	}
    	$wp_http_request_args = ['method', 'timeout', 'redirection', 'httpversion', 'user-agent', 'reject_unsafe_urls', 'blocking', 'headers', 'cookies', 'body', 'compress', 'decompress', 'sslverify', 'sslcertificates', 'stream', 'filename', 'limit_response_size'];
    	$seems_wp_http_request = true;
    	foreach(array_keys($args) as $arg){
    		if(!in_array($arg, $wp_http_request_args)){
    			$seems_wp_http_request = false;
    			break;
    		}
    	}
    	if(!$method_verification){
    		return $seems_wp_http_request;
    	}
    	if(empty($args['method'])){
    		return false;
    	}
    	return in_array($args['method'], ['DELETE', 'GET', 'HEAD', 'OPTIONS', 'PATCH', 'POST', 'PUT', 'TRACE']);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_seems_wp_error')){
    function ifwp_seems_wp_error($error = []){
		if(!ifwp_array_keys_exist(['code', 'data', 'message'], $error)){
			return false;
		}
		if(4 === count($error)){
			if(!array_key_exists('additional_errors', $error)){
				return false;
			}
		} else {
			if(3 !== count($error)){
				return false;
			}
		}
		if(!$error['code'] or !$error['message']){
			return false;
		}
		return new WP_Error($error['code'], $error['message'], $error['data']);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
