<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_sse_send')){
    function ifwp_sse_send($data = [], $event = 'message'){
        $uniqid = uniqid();
        $post_id = wp_insert_post([
            'post_title' => $uniqid,
			'post_type' => 'ifwp-sse',
        ], true);
        if(is_wp_error($post_id)){
            return $post_id;
        }
        update_post_meta($post_id, 'data', $data);
        update_post_meta($post_id, 'event', $event);
        return wp_update_post([
			'ID' => $post_id,
            'post_status' => 'private',
            'post_title' => $uniqid . ' ' . $event . ' ' . $post_id,
        ], true);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_support_sse')){
    function ifwp_support_sse($limit = 10){
		$dir = __remote_lib('https://github.com/byjg/jquery-sse/archive/refs/tags/v0.1.4.zip', 'jquery-sse-0.1.4');
		if(is_wp_error($dir)){
			return $dir;
		}
        ifwp_one('init', function(){
            register_post_type('ifwp-sse', [
                'labels' => ifwp_post_type_labels('Server-sent event', 'Server-sent events', false),
                'menu_icon ' => 'dashicons-database',
                'show_in_admin_bar' => false,
                'show_ui' => true,
                'supports' => ['title'],
            ]);
        });
        ifwp_one('parse_request', function(&$wp){
            global $wpdb;
            if('ifwp-sse-server' !== $wp->request){
    			return;
    		}
            nocache_headers();
            header('Content-Type: text/event-stream');
            if(!empty($_SERVER['HTTP_LAST_EVENT_ID'])){
                $last_event_id = (int) $_SERVER['HTTP_LAST_EVENT_ID'];
            } elseif(!empty($_GET['last_event_id'])){
                $last_event_id = (int) $_GET['last_event_id'];
            } else {
                $query = "SELECT ID FROM $wpdb->posts WHERE post_status = 'private' AND post_type = 'ifwp-sse' ORDER BY ID DESC LIMIT 2";
                $last_event_id = (int) $wpdb->get_var($query, 0, 1);
            }
            $query = "SELECT ID FROM $wpdb->posts WHERE post_status = 'private' AND post_type = 'ifwp-sse' ORDER BY ID DESC LIMIT 1";
            $last_id = (int) $wpdb->get_var($query);
            if($last_id > $last_event_id){
                if($limit){
                    $str = "SELECT ID FROM $wpdb->posts WHERE ID > %d AND post_status = 'private' AND post_type = 'ifwp-sse' ORDER BY ID ASC LIMIT %d";
                    $sql = $wpdb->prepare($str, $last_event_id, $limit);
                } else {
                    $str = "SELECT ID FROM $wpdb->posts WHERE ID > %d AND post_status = 'private' AND post_type = 'ifwp-sse' ORDER BY ID ASC";
                    $sql = $wpdb->prepare($str, $last_event_id);
                }
                $last_ids = $wpdb->get_col($sql);
                foreach($last_ids as $last_id){
                    $data = get_post_meta($last_id, 'data', true);
                    $event = get_post_meta($last_id, 'event', true);
                    $retry = 1000;
                    echo 'event: ' . $event . PHP_EOL;
            		echo 'data: ' . wp_json_encode($data) . PHP_EOL;
            		echo 'id: ' . $last_id . PHP_EOL;
            		echo 'retry: ' . $retry . PHP_EOL;
            		echo PHP_EOL;
                }
        	}
            ob_flush();
            flush();
    		exit;
        });
        ifwp_one('wp_enqueue_scripts', function() use($dir){
			wp_enqueue_script('jquery-sse', ifwp_download_url($dir . '/jquery.sse.min.js'), ['jquery'], '0.1.4', true);
            wp_localize_script('jquery-sse', 'ifwp_sse', [
    			'server_url' => site_url('ifwp-sse-server'),
    		]);
        });
		return true;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  jQuery(function(){
//    var ifwp_sse_server = jQuery.SSE(ifwp_sse.server_url, {
//      onMessage: function(e){
//        console.log(e.lastEventId);
//        console.log(jQuery.parseJSON(e.data));
//      }
//    });
//    ifwp_sse_server.start();
//  });
//
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  ifwp_sse_send([
//    'hello' => 'world',
//  ], 'message');
//
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
