<?php

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_api_url')){
    function ifwp_zoom_api_url($endpoint = ''){
        $base = 'https://api.zoom.us/v2';
        $endpoint = str_replace($base, '', $endpoint);
        $endpoint = ltrim($endpoint, '/');
        $endpoint = untrailingslashit($endpoint);
        $endpoint = trailingslashit($base) . $endpoint;
        return $endpoint;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_auth')){
    function ifwp_zoom_auth($api_key = '', $api_secret = ''){
        $missing = [];
        if(empty($api_key) and defined('IFWP_ZOOM_API_KEY')){
            $api_key = IFWP_ZOOM_API_KEY;
        }
        if(empty($api_secret) and defined('IFWP_ZOOM_API_SECRET')){
            $api_secret = IFWP_ZOOM_API_SECRET;
        }
        if(empty($api_key)){
            $missing[] = 'API Key';
        }
        if(empty($api_secret)){
            $missing[] = 'API Secret';
        }
        if(!empty($missing)){
            return ifwp_error(sprintf(__('Missing parameter(s): %s'), ifwp_implode_and($missing) . '.'));
        }
        $payload = [
            'exp' => time() + DAY_IN_SECONDS,
            'iss' => $api_key,
        ];
        $jwt = ifwp_jwt_encode($payload, $api_secret);
        IFWP::$zoom_jwt = $jwt;
        return $jwt;
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_delete')){
    function ifwp_zoom_delete($endpoint = '', $args = [], $timeout = 30){
        return ifwp_zoom_request('DELETE', $endpoint, $args, $timeout);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_get')){
    function ifwp_zoom_get($endpoint = '', $args = [], $timeout = 30){
        return ifwp_zoom_request('GET', $endpoint, $args, $timeout);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_patch')){
    function ifwp_zoom_patch($endpoint = '', $args = [], $timeout = 30){
        return ifwp_zoom_request('PATCH', $endpoint, $args, $timeout);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_post')){
    function ifwp_zoom_post($endpoint = '', $args = [], $timeout = 30){
        return ifwp_zoom_request('POST', $endpoint, $args, $timeout);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_put')){
    function ifwp_zoom_put($endpoint = '', $args = [], $timeout = 30){
        return ifwp_zoom_request('PUT', $endpoint, $args, $timeout);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if(!function_exists('ifwp_zoom_request')){
    function ifwp_zoom_request($method = '', $endpoint = '', $args = [], $timeout = 30){
        if(!isset(IFWP::$zoom_jwt)){
            IFWP::$zoom_jwt = '';
        }
        $url = ifwp_zoom_api_url($endpoint);
        if(!is_array($args)){
    		$args = wp_parse_args($args);
    	}
        $args = [
            'body' => $args,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Bearer ' . IFWP::$zoom_jwt,
                'Content-Type' => 'application/json',
            ],
            'timeout' => ifwp_sanitize_timeout($timeout),
        ];
        return ifwp_remote_request($method, $url, $args);
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
